/**
 * Student: Nicolae Alina-Elena
 * Grupa: 331 CA
 * Proiect: Hangman pe LCD
*/

#include "u8g.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>

#if defined(__AVR__)
#include <avr/interrupt.h>
#include <avr/io.h>
#endif

#define KEYPAD_PORT	 PORTB
#define KEYPAD_DDR   DDRB
#define KEYPAD_PIN   PINB

#define INDEX_WORD 3
#define NUM_LIVES  6
#define LCD_START_W 50

char* words[10]={"mecanic", "proiect", "mancare", "pointer", "revista",
						"elefant", "butoane", "placuta", "catelus", "avionas"};
char alphabet[26]={'a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z'};
char* cuvintel;
char* letters_guessed;
int game_won = 0;
int in_game = 0;

u8g_t u8g;

uint8_t GetKeyPressed(void);
void game_page(void);

// setare LCD
void u8g_setup(void) {
    u8g_Init8Bit(&u8g, &u8g_dev_st7920_128x64_8bit,
		// PORT DATE
        PN(0, 0), PN(0, 1), PN(0, 2), PN(0, 3), PN(0, 4), PN(0, 5), PN(0, 6), PN(0, 7),
		// ENABLE
		PN(2, 4),
        U8G_PIN_NONE,
        U8G_PIN_NONE,
		// RS
        PN(2, 2),
		// RW
        PN(2, 3),
		// RESET
        PN(2, 5));
}

void sys_init(void) {
#if defined(__AVR__)
    // select minimal prescaler (max system speed)
    CLKPR = 0x80;
    CLKPR = 0x00;
#endif
}

// initializare tastatura
void io_init(void) {
	// coloanele 3, 1, 5 => setate pe input
    KEYPAD_DDR &= ~((1 << PB2) | (1 << PB0) | (1 << PB4));
	// setare rezistente de pull-up pe 1
    KEYPAD_PORT |=  (1 << PB2) | (1 << PB0) | (1 << PB4);

	// randurile 2, 7, 6, 4 => setate pe input
	KEYPAD_DDR &= ~((1 << PB1) | (1 << PB6) | (1 << PB5) | (1 << PB3));
	// setare rezistente de pull-up pe 1
	KEYPAD_PORT &= ~((1 << PB1) | (1 << PB6) | (1 << PB5) | (1 << PB3));
}

// desenare pagina initiala a jocului
void drawFirstPage(void) {
    u8g_SetFont(&u8g, u8g_font_gdr9);

    u8g_DrawStr(&u8g, 7, 11, "   Hangman Game ");
    u8g_DrawStr(&u8g, 0, 31, " Press the # button ");
	u8g_DrawStr(&u8g, 0, 46, " to start the game.");
}

void end_game(void) {
	if (game_won == 1) { // a castigat
		u8g_FirstPage(&u8g);
		do {
			u8g_SetFont(&u8g, u8g_font_gdr9);
			u8g_DrawStr(&u8g, 30, 11, "Game won!");
			u8g_DrawStr(&u8g, 10, 26, "Congratulations!");
		} while (u8g_NextPage(&u8g));
		u8g_Delay(100);
	} else { // a pierdut
		u8g_FirstPage(&u8g);
		do {
			u8g_SetFont(&u8g, u8g_font_gdr9);
			u8g_DrawStr(&u8g, 30, 11, "Game over!");
		} while (u8g_NextPage(&u8g));
		u8g_Delay(100);
	}
}

uint8_t GetKeyPressed(void) {
    uint8_t r,c;
	uint8_t col[3] = {2, 0, 4};
	uint8_t row[4] = {1, 6, 5, 3};
	char buffer[33];


	for (c = 0; c < 3; c++) {
		for (r = 0; r < 4; r++) {
			KEYPAD_DDR &= ~((1 << PB1) | (1 << PB6) | (1 << PB5) | (1 << PB3));
			KEYPAD_PORT |= (1 << PB1) | (1 << PB6) | (1 << PB5) | (1 << PB3);
			KEYPAD_DDR |= (1 << row[r]);
			KEYPAD_PORT &= ~(1 << row[r]);
			_delay_ms(1);
			
			if((KEYPAD_PIN & (1 << col[c])) == 0) {
				return (r*3+c);
			}
		}
	}

	return 0XFF;//Indicate No key pressed
}

// functie care afiseaza caracterul oriunde il gaseste in cuvant
// returneaza nr de aparitii al caracterului in cuvant
int search(char c, char* cuvant, int length){

	int i, count = 0;
	char *buf = &c;
	uint8_t distance = 5;
	char buffer[33];
	
	for (i = 0; i < length; i++) {
		if (c == cuvant[i]) {
			cuvintel[i] = c;
			count++;
		}
	}

	return count;
}

// returneaza litera corespunzatoare tastei numerice apasate
char get_letter_from_key(uint8_t key) {
	char letter = 'a';
	uint8_t index;
	
	switch(key) {
		case 0:
			while(1) {
				letter = 'a';
				index = GetKeyPressed();
				if (index == 10) { // confirma -> tasta 0
					break;
				} else if (index == 9) { // tasta *
					letter = 'b';
					break;
				} else if (index == 11) {
					letter = 'c';
					break;
				}
			}
			break;
		case 1:
			while(1) {
				letter = 'd';
				index = GetKeyPressed();
				if (index == 10) { // confirma -> tasta 0
					break;
				} else if (index == 9) { // tasta *
					letter = 'e';
					break;
				} else if (index == 11) {
					letter = 'f';
					break;
				}
			}
			break;
		case 2:
			while(1) {
				letter = 'g';
				index = GetKeyPressed();
				if (index == 10) { // confirma -> tasta 0
					break;
				} else if (index == 9) { // tasta *
					letter = 'h';
					break;
				} else if (index == 11) {
					letter = 'i';
					break;
				}
			}
			break;
		case 3:
			while(1) {
				letter = 'j';
				index = GetKeyPressed();
				if (index == 10) { // confirma -> tasta 0
					break;
				} else if (index == 9) { // tasta *
					letter = 'k';
					break;
				} else if (index == 11) {
					letter = 'l';
					break;
				}
			}
			break;
		case 4:
			while(1) {
				letter = 'm';
				index = GetKeyPressed();
				if (index == 10) { // confirma -> tasta 0
					break;
				} else if (index == 9) { // tasta *
					letter = 'n';
					break;
				} else if (index == 11) {
					letter = 'o';
					break;
				}
			}
			break;
		case 5:
			while(1) {
				letter = 'p';
				index = GetKeyPressed();
				if (index == 10) { // confirma -> tasta 0
					break;
				} else if (index == 9) { // tasta *
					letter = 'q';
					break;
				} else if (index == 11) {
					letter = 'r';
					break;
				}
			}
			break;
		case 6:
			while(1) {
				letter = 's';
				index = GetKeyPressed();
				if (index == 10) { // confirma -> tasta 0
					break;
				} else if (index == 9) { // tasta *
					letter = 't';
					break;
				} else if (index == 11) {
					letter = 'u';
					break;
				}
			}
			break;
		case 7:
			while(1) {
				letter = 'v';
				index = GetKeyPressed();
				if (index == 10) { // confirma -> tasta 0
					break;
				} else if (index == 9) { // tasta *
					letter = 'w';
					break;
				} else if (index == 11) {
					letter = 'x';
					break;
				}
			}
			break;
		case 8:
			while(1) {
				letter = 'y';
				index = GetKeyPressed();
				if (index == 10) { // confirma -> tasta 0
					break;
				} else if (index == 9) { // tasta *
					letter = 'z';
					break;
				}
			}
			break;
	}
	
	return letter;
}

// cand se apasa o tasta se returneaza daca a fost ghicita sau nu
int guess_letter(uint8_t letter_index) {
	char* word = words[INDEX_WORD];
	int length = strlen(word);
	char letter;
	char buffer[33];
	
	if (letter_index != 0xFF) {
		letter = get_letter_from_key(letter_index);
		return search(letter, word, length);
	}

	return -1;
}

void drawManStick(int no_lives) {
	switch(no_lives) {
		case NUM_LIVES - 1:
			u8g_DrawStr(&u8g, 110, 26, "O");
			break;
		case NUM_LIVES - 2:
			u8g_DrawStr(&u8g, 110, 26, "O");
			u8g_DrawStr(&u8g, 113, 37, "|");
			u8g_DrawStr(&u8g, 113, 45, "|");
			break;
		case NUM_LIVES - 3:
			u8g_DrawStr(&u8g, 110, 26, "O");
			u8g_DrawStr(&u8g, 113, 37, "|");
			u8g_DrawStr(&u8g, 113, 45, "|");
			u8g_DrawStr(&u8g, 108, 35, "\\");
			break;
		case NUM_LIVES - 4:
			u8g_DrawStr(&u8g, 110, 26, "O");
			u8g_DrawStr(&u8g, 113, 37, "|");
			u8g_DrawStr(&u8g, 113, 45, "|");
			u8g_DrawStr(&u8g, 108, 35, "\\");
			u8g_DrawStr(&u8g, 114, 35, "/");
			break;
		case NUM_LIVES - 5:
			u8g_DrawStr(&u8g, 110, 26, "O");
			u8g_DrawStr(&u8g, 113, 37, "|");
			u8g_DrawStr(&u8g, 113, 45, "|");
			u8g_DrawStr(&u8g, 108, 35, "\\");
			u8g_DrawStr(&u8g, 114, 35, "/");
			u8g_DrawStr(&u8g, 108, 60, "/");
			break;
		case NUM_LIVES - 6:
			u8g_DrawStr(&u8g, 110, 26, "O");
			u8g_DrawStr(&u8g, 113, 37, "|");
			u8g_DrawStr(&u8g, 113, 45, "|");
			u8g_DrawStr(&u8g, 108, 35, "\\");
			u8g_DrawStr(&u8g, 114, 35, "/");
			u8g_DrawStr(&u8g, 108, 60, "/");
			u8g_DrawStr(&u8g, 114, 60, "\\");
			break;
	}
}

// se ghiceste cuvantul: contine mai multe apeluri de guess_letter
void guess_word(uint8_t key) {

	char* word = words[INDEX_WORD];
	int length = strlen(word);
	int num_letters_remained = length; // numarul de litere neghicite
	int no_lives = NUM_LIVES; // numarul de vieti initial
	int i;
	char buffer[33];

	game_won = 0; // initial e fals => necesar pt end_game

	while (1) {
		if (num_letters_remained == 0) { // a ghicit toate literele => a castigat
			game_won = 1;
			break;
		}
		else if (no_lives == 0) { // a ramas fara vieti => a pierdut
			game_won = 0;
			break;
		}

		if (key != 0xFF) {
			int letter_guessed = guess_letter(key);
			if (letter_guessed == 0) { // nu a fost ghicit
				no_lives--;
			}
			else if (letter_guessed != -1) {
				if (search(get_letter_from_key(key), cuvintel, length) != 0)
					num_letters_remained--;
			}
			u8g_FirstPage(&u8g);
			do {
				u8g_SetFont(&u8g, u8g_font_gdr9);
				u8g_DrawStr(&u8g, 0, 11, "    Choose a letter ");
				u8g_DrawStr(&u8g, 5, 26, "Lives:");
				u8g_DrawStr(&u8g, 50, 26, itoa(no_lives, buffer, 10));
				drawManStick(no_lives);
				uint8_t distance = 5;
				u8g_DrawStr(&u8g, distance, LCD_START_W, cuvintel);
			} while (u8g_NextPage(&u8g));
			u8g_Delay(100);
		}
		key = GetKeyPressed();
	}
	
	while(1)
		end_game();
}

void game_page(void) {
	int i;
	char* word = words[INDEX_WORD];
	int length = strlen(word);
	char buffer[33];
	uint8_t key = 0xFF;
	cuvintel = "*******";

	u8g_FirstPage(&u8g);
	do {
		u8g_SetFont(&u8g, u8g_font_gdr9);
		u8g_DrawStr(&u8g, 0, 11, "    Choose a letter ");
		uint8_t distance = 5;
		u8g_DrawStr(&u8g, distance, LCD_START_W, cuvintel);
		
		key = GetKeyPressed();
		if (key != 0xFF)
			break;
	} while (u8g_NextPage(&u8g));
	u8g_Delay(100);
	
	if (key != 0xFF)
		guess_word(key);

}

// controller-ul jocului
void start_game(void) {
	char* word = words[INDEX_WORD];
	int length = strlen(word);

	if (in_game == 1) {
		game_page();
		return;
	}
	
	u8g_FirstPage(&u8g);
	do {
		u8g_SetFont(&u8g, u8g_font_gdr9);
		u8g_DrawStr(&u8g, 0, 11, " For 'a' press 1 and 0 ");
		u8g_DrawStr(&u8g, 0, 26, " For 'b' press 1 and * ");
		u8g_DrawStr(&u8g, 0, 41, " For 'c' press 1 and # ");
		u8g_DrawStr(&u8g, 5, 56, "Press * to continue ");
		u8g_Delay(100);
		uint8_t key = GetKeyPressed();
		if (key == 9) { // *
			in_game = 1;
		}
	} while (u8g_NextPage(&u8g));
	u8g_Delay(100);
}

void light(int ok)
{
	DDRD |= 1 << PD7;
	if (ok)
		PORTD |= 1 << PD7;
	else
		PORTD &= ~(1 << PD7);
}


int main(void) {

	MCUCR |= (1 << JTD);
    MCUCR |= (1 << JTD);

	sys_init();
    u8g_setup();
	io_init();
	
	char buffer[33];

	while (1) {
		uint8_t key = GetKeyPressed();
		if (key == 11) // #
			break;
		u8g_FirstPage(&u8g);
		do {
			drawFirstPage();
		} while (u8g_NextPage(&u8g));
		u8g_Delay(100);
	}

	in_game = 0;
	while (1) {
		start_game();
	}

}